// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Authors: Antonio Carzaniga <carzanig@cs.colorado.edu>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: ft_allocator.h,v 1.2 2002/09/30 04:51:48 carzanig Exp $
//
#ifndef FT_ALLOCATOR_H
#define FT_ALLOCATOR_H

#include <cstddef>

class FTAllocator;

/** constructor operator that uses <code>FTAllocator</code> **/
extern void *		operator new (size_t s, FTAllocator &);
/** destructor operator that uses <code>FTAllocator</code> 
 *
 *  this is in fact a noop, since we never deallocate on a per-object
 *  basis when we use <code>FTAllocator</code>
 **/
extern void		operator delete (void *, size_t s, FTAllocator &);
/** constructor operator that uses <code>FTAllocator</code> **/
extern void *		operator new[] (size_t, FTAllocator &);
/** destructor operator that uses <code>FTAllocator</code> 
 *
 *  this is in fact a noop, since we never deallocate on a per-object
 *  basis when we use <code>FTAllocator</code>
 **/
extern void 		operator delete[] (void *, size_t s, FTAllocator &);

/** memory manager used by the forwarding table
 *
 *  This manager allocates and keeps track of blocks of memory.
 *  Individual segments of these blocks are then made available to
 *  users.  The manager does not keep track of each individual
 *  segment.  Rather it allows the user to deallocate all the memory
 *  at once.  This memory manager is ideal for "dictionary" type data
 *  structures that allow only growing structure.
 **/
class FTAllocator {
public:
    
    /** allocates a memory segment of the given size **/
    void *		allocate(size_t s);
    /** deallocates all the memory blocks used by this allocator **/
    void		clear();
    /** returns the total number of bytes used by this allocator **/
    size_t		size() const;

			FTAllocator(size_t bs);

private:
    /** memory block **/
    struct block {
	block *	next;
	char	bytes[0];
    };

    /** block size **/
    const size_t	BSize;

    /** pointer to the list of used blocks **/
    block *		blist;

    /** first available position within the current block **/
    size_t		free_pos;

    /** number of blocks in use **/
    size_t		bcount;
};

#ifdef HAVE_INLINE
#include "ft_allocator.icc"
#endif

#endif
